﻿'*********************************************************************
' modFelicaLib.vb で読み込んだ DLL の関数はそのままでは扱いづらいので
' 機能ごとにある程度まとめてみました。
'*********************************************************************
Imports System.Runtime.InteropServices
Imports UInt8 = System.Byte

'================================
' felicalib.dll アクセス用クラス
'================================
Public Class CFelicaLib
    Implements IDisposable 'デストラクタ

    '==========
    ' 変数定義
    '==========
    Private p_ptr As IntPtr  'Pasoriポインタ
    Private f_ptr As IntPtr  'felica構造体ポインタ

    '===========================================
    '【関数名】New
    '【引  数】なし
    '【戻り値】なし
    '-------------------------------------------
    ' コンストラクタ。ポインタの初期化を行なう。
    '===========================================
    Public Sub New()
        '初期化
        p_ptr = IntPtr.Zero
        f_ptr = IntPtr.Zero
    End Sub

    '=================================
    '【関数名】Dispose
    '【引  数】なし
    '【戻り値】なし
    '---------------------------------
    ' デストラクタ。PaSoRiを解放する。
    '=================================
    Public Sub Dispose() Implements IDisposable.Dispose
        'PaSoRi の接続を解放
        Pasori_Free()
    End Sub

    '===============================================
    '【関数名】Pasori_Connect
    '【引  数】なし
    '【戻り値】[out] Boolean  TRUE   PaSoRi接続成功
    '                         FALSE  PaSoRi接続失敗
    '-----------------------------------------------
    ' PaSoRiに接続して使用可能な状態にする。
    '===============================================
    Public Function Pasori_Connect() As Boolean
        Try
            '----------------------
            ' PaSoRi ハンドル取得
            '----------------------
            p_ptr = pasori_open(Nothing)
            If p_ptr = IntPtr.Zero Then
                Return False
            End If

            '---------------------
            ' PaSoRi 初期化(接続)
            '---------------------
            If pasori_init(p_ptr) <> 0 Then
                Return False
            End If

            Return True
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Pasori_Connect()")
            Return False
        End Try
    End Function

    '======================
    '【関数名】Pasori_Free
    '【引  数】なし
    '【戻り値】なし
    '----------------------
    ' PaSoRiの接続を解放
    '======================
    Public Sub Pasori_Free()
        Try
            If f_ptr <> IntPtr.Zero Then felica_free(f_ptr)
            If p_ptr <> IntPtr.Zero Then pasori_close(p_ptr)
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Pasori_Free()")
        End Try
    End Sub

    '===========================================================
    '【関数名】Polling
    '【引  数】なし
    '【戻り値】[out] Boolean  TRUE   felicaハンドルの取得に成功
    '                         FALSE  felicaハンドルの取得に失敗
    '-----------------------------------------------------------
    ' ポーリング。FeliCaの読み取り準備。
    '===========================================================
    Public Function Polling() As Boolean
        Try
            '--------------------------------
            ' felicaハンドルを一度クリアする
            '--------------------------------
            If f_ptr <> IntPtr.Zero Then
                felica_free(f_ptr)
            End If

            '------------
            ' ポーリング
            '------------
            f_ptr = felica_polling(p_ptr, POLLING_ANY, 0, 0)

            '------------
            ' 結果を返す
            '------------
            If f_ptr = IntPtr.Zero Then
                Return False
            Else
                Return True
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Polling()")
            Return False
        End Try
    End Function

    '====================================
    '【関数名】getIDm
    '【引  数】なし
    '【戻り値】[out] String  FeliCaのIDm
    '------------------------------------
    ' FeliCaのIDmを取得する。
    '====================================
    Public Function getIDm() As String
        Try
            '------------
            ' エラー処理
            '------------
            If f_ptr = IntPtr.Zero Then
                Return ""
            End If

            '----------
            ' 変数定義
            '----------
            Dim IDm As String
            Dim buf(8) As UInt8

            '-------------------------
            ' 変数bufのアドレスを取得
            '-------------------------
            Dim gch As GCHandle = GCHandle.Alloc(buf, GCHandleType.Pinned)
            Dim b As IntPtr = gch.AddrOfPinnedObject().ToInt32

            '-------------
            ' IDm読み取り
            '-------------
            felica_getidm(f_ptr, b)
            IDm = hexdump(buf, 8)

            '--------------
            ' メモリの解放
            '--------------
            gch.Free()

            '------------
            ' 結果を返す
            '------------
            Return IDm
        Catch ex As Exception
            MessageBox.Show(ex.Message, "getIDm()")
            Return ""
        End Try
    End Function

    '====================================
    '【関数名】getPMm
    '【引  数】なし
    '【戻り値】[out] String  FeliCaのPMm
    '------------------------------------
    ' FeliCaのPMmを取得する。
    '====================================
    Public Function getPMm() As String
        Try
            '------------
            ' エラー処理
            '------------
            If f_ptr = IntPtr.Zero Then
                Return ""
            End If

            '----------
            ' 変数定義
            '----------
            Dim PMm As String
            Dim buf(8) As UInt8

            '-------------------------
            ' 変数bufのアドレスを取得
            '-------------------------
            Dim gch As GCHandle = GCHandle.Alloc(buf, GCHandleType.Pinned)
            Dim b As IntPtr = gch.AddrOfPinnedObject().ToInt32

            '-------------
            ' PMm読み取り
            '-------------
            felica_getpmm(f_ptr, b)
            PMm = hexdump(buf, 8)

            '--------------
            ' メモリの解放
            '--------------
            gch.Free()

            '------------
            ' 結果を返す
            '------------
            Return PMm
        Catch ex As Exception
            MessageBox.Show(ex.Message, "getPMm()")
            Return ""
        End Try
    End Function

    '======================================
    '【関 数 名】getIDmPMm
    '【第１引数】[out] String  FeliCaのIDm
    '【第２引数】[out] String  FeliCaのPMm
    '【戻 り 値】なし
    '--------------------------------------
    ' FeliCaのIDmとPMmを同時に取得する。
    '======================================
    Public Sub getIDmPMm(ByRef IDm As String, ByRef PMm As String)
        Try
            '------------
            ' エラー処理
            '------------
            If f_ptr = IntPtr.Zero Then
                IDm = ""
                PMm = ""
                Return
            End If

            '--------------------------
            ' felica構造体の実体を取得
            '--------------------------
            Dim f As felica = Marshal.PtrToStructure(f_ptr, GetType(felica))

            '------------------
            ' IDm・PMm読み取り
            '------------------
            IDm = hexdump(f.IDm, 8)
            PMm = hexdump(f.PMm, 8)
        Catch ex As Exception
            MessageBox.Show(ex.Message, "getIDmPMm()")
            IDm = ""
            PMm = ""
        End Try
    End Sub

    '=============================================
    '【関 数 名】hexdump
    '【第１引数】[in]  UInt8()  データ配列
    '【第２引数】[in]  Integer  配列のサイズ
    '【戻 り 値】[out] String   16進数文字列
    '---------------------------------------------
    ' 受け取ったデータを16進数の文字列に変換する。
    '=============================================
    Private Function hexdump(ByVal arg() As UInt8, ByVal size As Integer) As String
        Dim sbResult As New System.Text.StringBuilder

        For I As Integer = 0 To size - 1
            sbResult.Append(arg(I).ToString("X2"))
        Next

        Return sbResult.ToString()
    End Function
End Class
